"""StrUtils - String manipulation routines

Contains routines  Amt2Str, Str2Amt and Str2Percent which convert
commonly used numeric formats from strings to numbers and vice-versa

	Sudhir Shenoy, Nov. 1995"""

#### Copyright (C) 1995-96, Sudhir Shenoy.
####
#### This library is free software; you can redistribute it and/or
#### modify it under the terms of the GNU Library General Public
#### License as published by the Free Software Foundation; either
#### version 2 of the License, or (at your option) any later version.
#### 
#### This library is distributed in the hope that it will be useful,
#### but WITHOUT ANY WARRANTY; without even the implied warranty of
#### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#### Library General Public License for more details.
#### 
#### You should have received a copy of the GNU Library General Public
#### License along with this library; if not, write to the
#### Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
#### MA 02139, USA.

import string, regex, regsub
from types import TupleType

numpat1 = regex.compile('[0-9]*\.[0-9]+\(e\([+-]\)?[0-9]+\)?')
numpat2 = regex.compile('[0-9]+\.[0-9]*\(e\([+-]\)?[0-9]+\)?')
numpat3 = regex.compile('[0-9]+\(e\([+-]\)?[0-9]+\)?')
numpat4 = regex.compile('\([0-9]?[0-9]?[0-9],?\)?\([0-9][0-9][0-9],?\)+\(\.[0-9]*\)?')

def Str2Amt(s):
    """Convert a string to a number.
    The string may have commas and multipliers at the end - 'k', 'm' or 'b'
    corresponding to 1000, 1 million and 1 billion repectively.
    	1.23M => 1230000
	4,567K => 4567000"""

    s = string.strip(string.lower(s))
    if len(s) < 1:
	return 0
    try:
	return string.atof(s)
    except ValueError:
	pass

    mult = 1.0
    while s[-1] in ['k', 'm', 'b']:
	if s[-1] == 'k':
	    mult = mult * 1000.0
	elif s[-1] == 'm':
	    mult = mult * 1000000.0
	else:				# == 'b'
	    mult = mult * 1000000000.0
	s = s[:-1]
    l = len(s)
    if numpat1.match(s) == l or numpat2.match(s) == l or numpat3.match(s) == l:
	# scientific format
	return mult * string.atof(s)
    elif numpat4.match(s) == l:
	s = regsub.gsub(',', '', s)
	return mult * string.atof(s)

    raise ValueError, "Invalid amount string"

def Amt2Str(amt, decimals = -1, maxlen = 0, use_abbrevs = 1):
    """Convert a number to a string
    If decimals is a non-negative integer, the number is rounded to the
    desired precision. Commas are inserted every three digits to the left
    of the decimal point. If the number ends in zeros, the zeros are replaced
    by 'K', 'M' or 'B' depending on the number of zeros"""

    a = amt
    if decimals > -1:
	a = round(amt, decimals)
    prefix = ''
    if amt < 0:
	prefix = '-'
	amt = -amt
    s = str(a)
    p = string.splitfields(s, '.')
    num = p[0]
    dec = None
    if len(p) > 1:
	dec = p[1]
    output = ''
    while len(num) > 3:
	output = num[-3:] + ',' + output
	num = num[:-3]
    if len(num):
	output = num + ',' + output
    output = output[:-1]		# remove trailing comma
    if dec:
	output = output + '.' + dec
	while output[-1] == '0':
	    output = output[:-1]
	if output[-1] == '.':
	    output = output[:-1]

    if not output:
	output = '0'

    if use_abbrevs:
	suffix = ''
	while output[-4:] == ',000':
	    output = output[:-4]
	    suffix = suffix + 'K'
	output = output + suffix
	output = regsub.gsub('KKK', 'B', output)
	output = regsub.gsub('KK', 'M', output)

    return (prefix + output)

def Str2Percent(s):
    idx = string.find(s, '%')
    if idx < 0:
	raise ValueError, "Invalid percentage string"
    s = string.strip(s[:idx])
    amt = Str2Amt(s)
    amt = amt / 100.0
    return amt

# end of StrUtils.py
