#! /usr/local/bin/python
#
# tkxtrademo.py --
#	This is a demo of TkXtra.py - a collection of enhanced Tk widgets
#	written in Python. It shows off most of the TkXtra widgets but
#	*not* all of them.
#
#	- Sudhir Shenoy, 1996.

import os, sys, TkXtra
tkx = TkXtra				# to save weary fingers

WELCOME_TEXT = """This is a demonstration of the widgets in TkXtra.py
The advantage of using TkXtra instead of (say) Tix is that TkXtra is
written entirely in Python and depends only upon Tkinter. As long as
you have a working version of Tkinter, TkXtra will work.
The two ComboBox widgets on the left are examples of TkXtra widgets as
is this whole window which is an example of a PanedWindow widget."""

DEMO_PROGS = {
    'Xman like program' : 'pyman',
    'Scrolled List': 'scrolled_list',
    'Scrolled Text': 'scrolled_text',
    'Paned Windows': 'paned_window',
    'Form (various widgets)': 'form'
}

class Demo:
    def __init__(self):
	progname = sys.argv[0]
	dirname = os.path.dirname(progname)
	if dirname and dirname != os.curdir:
	    self.dir = dirname
	    index = -1
	    for i in range(len(sys.path)):
		p = sys.path[i]
		if p in ("", os.curdir):
		    index = i
	    if index >= 0:
		sys.path[index] = dirname
	    else:
		sys.path.insert(0, dirname)
	else:
	    self.dir = os.getcwd()
	sys.path.insert(0, self.dir + '/samples')

	self.make_interface()
	self.list_demo_progs()
	self.root.mainloop()

    def exit(self, event=None):
	sys.exit(0)

    def make_menu(self, top):
	w = tkx.Frame(top, bd=2, relief=tkx.RAISED)
	file = tkx.Menubutton(w, text='File', underline=0, takefocus=0)
	file.pack(side=tkx.LEFT)
	fm = tkx.Menu(file)
	file['menu'] = fm
	fm.add_command(label='Exit', underline=1, accelerator='Ctrl+X',
		       command=self.exit)
	top.bind_all("<Control-x>", self.exit)
	return w

    def make_demo_area(self, top):
	pw = tkx.PanedWindow(self.root)
	pw.add_pane('welcome', min_size=20)
	pw.add_pane('main', min_size=100)
	f1 = pw.pane('welcome')
	ff = tkx.Frame(f1)
	c1 = tkx.ComboBox(ff, label='Font: ', variable=self.welcome_font,
			  command=self.show_welcome,
			  opt_list=['Courier','Times','Helvetica','Lucida'],
			  init_val='Helvetica')
	c2 = tkx.ComboBox(ff, label='Size: ', variable=self.welcome_size,
			  command=self.show_welcome,
			  opt_list=['8', '10', '12', '14', '18'],
			  init_val='14')
	c1.pack(side=tkx.LEFT)
	c2.pack(side=tkx.LEFT)
	ff.pack(side=tkx.LEFT, expand=0)
	self.welcome_msg = tkx.Label(f1, text=WELCOME_TEXT)
	self.welcome_msg.pack(side=tkx.BOTTOM, anchor='n', fill=tkx.BOTH, expand=1)

	f2 = pw.pane('main')
	f21 = tkx.Frame(f2)
	f21.pack(side=tkx.LEFT, expand=1, fill=tkx.Y)
	self.prog_listbox = tkx.ScrolledList(f21, scrollbar=tkx.AUTO,
					     dbl_click=self.run_program)
	self.prog_listbox.pack(side=tkx.TOP, expand=1, fill=tkx.Y)
	self.listbox = self.prog_listbox.listbox
	btns = tkx.ButtonBox(f21, orient=tkx.HORIZONTAL)
	btns.add(text='Show Source ...', command=self.show_source)
	btns.add(text='Run ...', command=self.run_program1)
	btns.pack(side=tkx.BOTTOM, expand=0, fill=tkx.X)

	self.prog_text = tkx.ScrolledText(f2)
	self.prog_text.pack(side=tkx.RIGHT, expand=1, fill=tkx.BOTH)

	return pw

    def make_interface(self):
	tkx.Tk().withdraw()
	self.root = tkx.Toplevel()
	self.root.title('TkXtra Widget Demo')
	self.root.geometry('830x620+100+100')

	self.welcome_font = tkx.StringVar()
	self.welcome_size = tkx.StringVar()

	me = self.make_menu(self.root)
	da = self.make_demo_area(self.root)

	me.pack(side=tkx.TOP, fill=tkx.X)
	da.pack(side=tkx.TOP, fill=tkx.BOTH, expand=1)

    def list_demo_progs(self):
	lst = DEMO_PROGS.keys()
	lst.sort()
	self.prog_listbox.add_list(lst)

    def run_program(self, progname):
	prog = DEMO_PROGS[progname]
	exec('import ' + prog)
	w = tkx.Toplevel()
	w.title(progname)
	rtn = eval(prog + '.RunSample')
	rtn(w)

    def run_program1(self, event=None):
	progname = self.listbox.get(self.listbox.curselection())
	self.run_program(progname)

    def show_source(self, event=None):
	progname = self.listbox.get(self.listbox.curselection())
	self.prog_text.text.delete('0.0', tkx.END)
	try:
	    f = open(self.dir + '/samples/' + DEMO_PROGS[progname] + '.py')
	    lines = f.readlines()
	    for s in lines:
		self.prog_text.text.insert(tkx.END, s)
	    f.close()
	except:
	    pass

    def show_welcome(self, *args, **kw):
	font = self.welcome_font.get()
	point = self.welcome_size.get()
	fontstr = '-*-%s-bold-r-normal-*-%s-*-*-*-*-*-*-*' % (font, point)
	self.welcome_msg['font'] = fontstr
	return 1

if __name__ == '__main__':
    demo = Demo()
